@@ -6,6 +6,8 @@ use p2p::{
 };
 use tokio_util::codec::length_delimited;
 
+const LASTEST_VERSION: &str = "2";
+
 /// All supported protocols
 ///
 /// The underlying network of CKB is flexible and complex. The flexibility lies in that it can support any number of protocols.
@@ -28,7 +30,7 @@ pub enum SupportProtocols {
     Identify,
     /// Feeler: used to detect whether the address is valid.
     ///
-    /// [RFC](https://github.com/nervosnetwork/rfcs/blob/v2020.01.15/rfcs/0007-scoring-system-and-network-security/0007-scoring-system-and-network-security.md#feeler-connection)
+    /// [RFC](https://github.com/nervosnetwork/rfcs/blob/master/rfcs/0007-scoring-system-and-network-security/0007-scoring-system-and-network-security.md#feeler-connection)
     /// [Eclipse Attacks on Bitcoin's Peer-to-Peer Network](https://cryptographylab.bitbucket.io/slides/Eclipse%20Attacks%20on%20Bitcoin%27s%20Peer-to-Peer%20Network.pdf)
     Feeler,
     /// Disconnect message: used to give the remote node a debug message when the node decides to disconnect.
@@ -42,6 +44,8 @@ pub enum SupportProtocols {
     ///
     /// [RFC](https://github.com/nervosnetwork/rfcs/blob/master/rfcs/0004-ckb-block-sync/0004-ckb-block-sync.md#new-block-announcement)
     Relay,
+    /// New Relay
+    RelayV2,
     /// Time: A protocol used for node pairing that warns if there is a large gap between the local time and the remote node.
     Time,
     /// Alert: A protocol reserved by the Nervos Foundation to publish network-wide announcements.
@@ -61,6 +65,7 @@ impl SupportProtocols {
             SupportProtocols::Sync => 100,
             SupportProtocols::Relay => 101,
             SupportProtocols::Time => 102,
+            SupportProtocols::RelayV2 => 103,
             SupportProtocols::Alert => 110,
         }
         .into()
@@ -76,6 +81,7 @@ impl SupportProtocols {
             SupportProtocols::DisconnectMessage => "/ckb/disconnectmsg",
             SupportProtocols::Sync => "/ckb/syn",
             SupportProtocols::Relay => "/ckb/rel",
+            SupportProtocols::RelayV2 => "/ckb/relay",
             SupportProtocols::Time => "/ckb/tim",
             SupportProtocols::Alert => "/ckb/alt",
         }
@@ -87,16 +93,21 @@ impl SupportProtocols {
         // we didn't invoke MetaBuilder#support_versions fn for these protocols (Ping/Discovery/Identify/Feeler/DisconnectMessage)
         // in previous code, so the default 0.0.1 value is used ( https://github.com/nervosnetwork/tentacle/blob/master/src/builder.rs#L312 )
         // have to keep 0.0.1 for compatibility...
+        //
+        // Here you have to make sure that the list of supported versions is sorted from smallest to largest
         match self {
-            SupportProtocols::Ping => vec!["0.0.1".to_owned()],
-            SupportProtocols::Discovery => vec!["0.0.1".to_owned()],
-            SupportProtocols::Identify => vec!["0.0.1".to_owned()],
-            SupportProtocols::Feeler => vec!["0.0.1".to_owned()],
-            SupportProtocols::DisconnectMessage => vec!["0.0.1".to_owned()],
-            SupportProtocols::Sync => vec!["1".to_owned()],
+            SupportProtocols::Ping => vec!["0.0.1".to_owned(), LASTEST_VERSION.to_owned()],
+            SupportProtocols::Discovery => vec!["0.0.1".to_owned(), LASTEST_VERSION.to_owned()],
+            SupportProtocols::Identify => vec!["0.0.1".to_owned(), LASTEST_VERSION.to_owned()],
+            SupportProtocols::Feeler => vec!["0.0.1".to_owned(), LASTEST_VERSION.to_owned()],
+            SupportProtocols::DisconnectMessage => {
+                vec!["0.0.1".to_owned(), LASTEST_VERSION.to_owned()]
+            }
+            SupportProtocols::Sync => vec!["1".to_owned(), LASTEST_VERSION.to_owned()],
             SupportProtocols::Relay => vec!["1".to_owned()],
-            SupportProtocols::Time => vec!["1".to_owned()],
-            SupportProtocols::Alert => vec!["1".to_owned()],
+            SupportProtocols::Time => vec!["1".to_owned(), LASTEST_VERSION.to_owned()],
+            SupportProtocols::Alert => vec!["1".to_owned(), LASTEST_VERSION.to_owned()],
+            SupportProtocols::RelayV2 => vec![LASTEST_VERSION.to_owned()],
         }
     }
 
@@ -109,7 +120,7 @@ impl SupportProtocols {
             SupportProtocols::Feeler => 1024,            // 1   KB
             SupportProtocols::DisconnectMessage => 1024, // 1   KB
             SupportProtocols::Sync => 2 * 1024 * 1024,   // 2   MB
-            SupportProtocols::Relay => 4 * 1024 * 1024,  // 4   MB
+            SupportProtocols::Relay | SupportProtocols::RelayV2 => 4 * 1024 * 1024, // 4   MB
             SupportProtocols::Time => 1024,              // 1   KB
             SupportProtocols::Alert => 128 * 1024,       // 128 KB
         }
@@ -129,7 +140,7 @@ impl SupportProtocols {
                 no_blocking_flag.disable_all();
                 no_blocking_flag
             }
-            SupportProtocols::Sync | SupportProtocols::Relay => {
+            SupportProtocols::Sync | SupportProtocols::Relay | SupportProtocols::RelayV2 => {
                 let mut blocking_recv_flag = BlockingFlag::default();
                 blocking_recv_flag.disable_connected();
                 blocking_recv_flag.disable_disconnected();
